@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2016 David Robillard <http://drobilla.net>
+  Copyright 2007-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -14,13 +14,19 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
+#include "lilv_internal.h"
+
+#include "lilv/lilv.h"
+#include "lv2/core/lv2.h"
+#include "serd/serd.h"
+
+#include <stdbool.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "lilv_internal.h"
-
-LILV_API LilvInstance*
+LilvInstance*
 lilv_plugin_instantiate(const LilvPlugin*        plugin,
                         double                   sample_rate,
                         const LV2_Feature*const* features)
@@ -42,7 +48,7 @@ lilv_plugin_instantiate(const LilvPlugin*        plugin,
 
 	LilvLib* lib = lilv_lib_open(plugin->world, lib_uri, bundle_path, features);
 	if (!lib) {
-		lilv_free(bundle_path);
+		serd_free(bundle_path);
 		return NULL;
 	}
 
@@ -76,7 +82,7 @@ lilv_plugin_instantiate(const LilvPlugin*        plugin,
 	}
 
 	free(local_features);
-	lilv_free(bundle_path);
+	serd_free(bundle_path);
 
 	if (result) {
 		if (result->lv2_handle == NULL) {
@@ -95,7 +101,7 @@ lilv_plugin_instantiate(const LilvPlugin*        plugin,
 	return result;
 }
 
-LILV_API void
+void
 lilv_instance_free(LilvInstance* instance)
 {
 	if (!instance) {
