@@ -1,5 +1,5 @@
 /*
-  Copyright 2012-2014 David Robillard <http://drobilla.net>
+  Copyright 2012-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -16,6 +16,18 @@
 
 #include "lilv_internal.h"
 
+#include "lilv/lilv.h"
+#include "lv2/core/lv2.h"
+#include "serd/serd.h"
+#include "zix/tree.h"
+
+#ifndef _WIN32
+#	include <dlfcn.h>
+#endif
+
+#include <stdint.h>
+#include <stdlib.h>
+
 LilvLib*
 lilv_lib_open(LilvWorld*               world,
               const LilvNode*          uri,
@@ -33,7 +45,8 @@ lilv_lib_open(LilvWorld*               world,
 	}
 
 	const char* const lib_uri  = lilv_node_as_uri(uri);
-	char* const       lib_path = lilv_file_uri_parse(lib_uri, NULL);
+	char* const       lib_path = (char*)serd_file_uri_parse(
+		(const uint8_t*)lib_uri, NULL);
 	if (!lib_path) {
 		return NULL;
 	}
@@ -42,7 +55,7 @@ lilv_lib_open(LilvWorld*               world,
 	void* lib = dlopen(lib_path, RTLD_NOW);
 	if (!lib) {
 		LILV_ERRORF("Failed to open library %s (%s)\n", lib_path, dlerror());
-		lilv_free(lib_path);
+		serd_free(lib_path);
 		return NULL;
 	}
 
@@ -56,18 +69,19 @@ lilv_lib_open(LilvWorld*               world,
 	if (ldf) {
 		desc = ldf(bundle_path, features);
 		if (!desc) {
-			LILV_ERRORF("Call to `lv2_lib_descriptor' in %s failed\n", lib_path);
-			lilv_free(lib_path);
+			LILV_ERRORF("Call to %s:lv2_lib_descriptor failed\n", lib_path);
+			dlclose(lib);
+			serd_free(lib_path);
 			return NULL;
 		}
 	} else if (!df) {
 		LILV_ERRORF("No `lv2_descriptor' or `lv2_lib_descriptor' in %s\n",
 		            lib_path);
 		dlclose(lib);
-		lilv_free(lib_path);
+		serd_free(lib_path);
 		return NULL;
 	}
-	lilv_free(lib_path);
+	serd_free(lib_path);
 
 	LilvLib* llib = (LilvLib*)malloc(sizeof(LilvLib));
 	llib->world          = world;
@@ -87,8 +101,7 @@ lilv_lib_get_plugin(LilvLib* lib, uint32_t index)
 {
 	if (lib->lv2_descriptor) {
 		return lib->lv2_descriptor(index);
-	}
-	if (lib->desc) {
+	} else if (lib->desc) {
 		return lib->desc->get_plugin(lib->desc->handle, index);
 	}
 	return NULL;
