@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2016 David Robillard <http://drobilla.net>
+  Copyright 2007-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -14,13 +14,20 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
+#include "filesystem.h"
+#include "lilv_internal.h"
+
+#include "lilv/lilv.h"
+#include "serd/serd.h"
+#include "sord/sord.h"
+
 #include <math.h>
+#include <stdbool.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "lilv_internal.h"
-
 static void
 lilv_node_set_numerics_from_string(LilvNode* val)
 {
@@ -145,13 +152,13 @@ lilv_node_new_from_node(LilvWorld* world, const SordNode* node)
 	return result;
 }
 
-LILV_API LilvNode*
+LilvNode*
 lilv_new_uri(LilvWorld* world, const char* uri)
 {
 	return lilv_node_new(world, LILV_VALUE_URI, uri);
 }
 
-LILV_API LilvNode*
+LilvNode*
 lilv_new_file_uri(LilvWorld* world, const char* host, const char* path)
 {
 	char*    abs_path = lilv_path_absolute(path);
@@ -164,42 +171,48 @@ lilv_new_file_uri(LilvWorld* world, const char* host, const char* path)
 	return ret;
 }
 
-LILV_API LilvNode*
+LilvNode*
 lilv_new_string(LilvWorld* world, const char* str)
 {
 	return lilv_node_new(world, LILV_VALUE_STRING, str);
 }
 
-LILV_API LilvNode*
+LilvNode*
 lilv_new_int(LilvWorld* world, int val)
 {
 	char str[32];
 	snprintf(str, sizeof(str), "%d", val);
 	LilvNode* ret = lilv_node_new(world, LILV_VALUE_INT, str);
-	ret->val.int_val = val;
+	if (ret) {
+		ret->val.int_val = val;
+	}
 	return ret;
 }
 
-LILV_API LilvNode*
+LilvNode*
 lilv_new_float(LilvWorld* world, float val)
 {
 	char str[32];
 	snprintf(str, sizeof(str), "%f", val);
 	LilvNode* ret = lilv_node_new(world, LILV_VALUE_FLOAT, str);
-	ret->val.float_val = val;
+	if (ret) {
+		ret->val.float_val = val;
+	}
 	return ret;
 }
 
-LILV_API LilvNode*
+LilvNode*
 lilv_new_bool(LilvWorld* world, bool val)
 {
 	LilvNode* ret = lilv_node_new(world, LILV_VALUE_BOOL,
 	                              val ? "true" : "false");
-	ret->val.bool_val = val;
+	if (ret) {
+		ret->val.bool_val = val;
+	}
 	return ret;
 }
 
-LILV_API LilvNode*
+LilvNode*
 lilv_node_duplicate(const LilvNode* val)
 {
 	if (!val) {
@@ -214,7 +227,7 @@ lilv_node_duplicate(const LilvNode* val)
 	return result;
 }
 
-LILV_API void
+void
 lilv_node_free(LilvNode* val)
 {
 	if (val) {
@@ -223,7 +236,7 @@ lilv_node_free(LilvNode* val)
 	}
 }
 
-LILV_API bool
+bool
 lilv_node_equals(const LilvNode* value, const LilvNode* other)
 {
 	if (value == NULL && other == NULL) {
@@ -251,7 +264,7 @@ lilv_node_equals(const LilvNode* value, const LilvNode* other)
 	return false; /* shouldn't get here */
 }
 
-LILV_API char*
+char*
 lilv_node_get_turtle_token(const LilvNode* value)
 {
 	const char* str    = (const char*)sord_node_get_string(value->node);
@@ -277,24 +290,26 @@ lilv_node_get_turtle_token(const LilvNode* value)
 		break;
 	case LILV_VALUE_INT:
 		node   = serd_node_new_integer(value->val.int_val);
-		result = (char*)node.buf;
+		result = lilv_strdup((char*)node.buf);
+		serd_node_free(&node);
 		break;
 	case LILV_VALUE_FLOAT:
 		node   = serd_node_new_decimal(value->val.float_val, 8);
-		result = (char*)node.buf;
+		result = lilv_strdup((char*)node.buf);
+		serd_node_free(&node);
 		break;
 	}
 
 	return result;
 }
 
-LILV_API bool
+bool
 lilv_node_is_uri(const LilvNode* value)
 {
 	return (value && value->type == LILV_VALUE_URI);
 }
 
-LILV_API const char*
+const char*
 lilv_node_as_uri(const LilvNode* value)
 {
 	return (lilv_node_is_uri(value)
@@ -302,13 +317,13 @@ lilv_node_as_uri(const LilvNode* value)
 	        : NULL);
 }
 
-LILV_API bool
+bool
 lilv_node_is_blank(const LilvNode* value)
 {
 	return (value && value->type == LILV_VALUE_BLANK);
 }
 
-LILV_API const char*
+const char*
 lilv_node_as_blank(const LilvNode* value)
 {
 	return (lilv_node_is_blank(value)
@@ -316,7 +331,7 @@ lilv_node_as_blank(const LilvNode* value)
 	        : NULL);
 }
 
-LILV_API bool
+bool
 lilv_node_is_literal(const LilvNode* value)
 {
 	if (!value) {
@@ -334,37 +349,37 @@ lilv_node_is_literal(const LilvNode* value)
 	}
 }
 
-LILV_API bool
+bool
 lilv_node_is_string(const LilvNode* value)
 {
 	return (value && value->type == LILV_VALUE_STRING);
 }
 
-LILV_API const char*
+const char*
 lilv_node_as_string(const LilvNode* value)
 {
 	return value ? (const char*)sord_node_get_string(value->node) : NULL;
 }
 
-LILV_API bool
+bool
 lilv_node_is_int(const LilvNode* value)
 {
 	return (value && value->type == LILV_VALUE_INT);
 }
 
-LILV_API int
+int
 lilv_node_as_int(const LilvNode* value)
 {
 	return lilv_node_is_int(value) ? value->val.int_val : 0;
 }
 
-LILV_API bool
+bool
 lilv_node_is_float(const LilvNode* value)
 {
 	return (value && value->type == LILV_VALUE_FLOAT);
 }
 
-LILV_API float
+float
 lilv_node_as_float(const LilvNode* value)
 {
 	if (lilv_node_is_float(value)) {
@@ -375,19 +390,19 @@ lilv_node_as_float(const LilvNode* value)
 	return NAN;
 }
 
-LILV_API bool
+bool
 lilv_node_is_bool(const LilvNode* value)
 {
 	return (value && value->type == LILV_VALUE_BOOL);
 }
 
-LILV_API bool
+bool
 lilv_node_as_bool(const LilvNode* value)
 {
 	return lilv_node_is_bool(value) ? value->val.bool_val : false;
 }
 
-LILV_API char*
+char*
 lilv_node_get_path(const LilvNode* value, char** hostname)
 {
 	if (lilv_node_is_uri(value)) {
