@@ -14,9 +14,6 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#define __STDC_LIMIT_MACROS
-
-#include "lilv_config.h"
 #include "lilv_internal.h"
 
 #include "lilv/lilv.h"
@@ -29,7 +26,6 @@
 
 #ifdef LILV_DYN_MANIFEST
 #    include "lv2/dynmanifest/dynmanifest.h"
-#    include <dlfcn.h>
 #endif
 
 #include <math.h>
@@ -356,7 +352,7 @@ lilv_plugin_load_ports_if_necessary(const LilvPlugin* const_plugin)
 		// Check sanity
 		for (uint32_t i = 0; i < plugin->num_ports; ++i) {
 			if (!plugin->ports[i]) {
-				LILV_ERRORF("Plugin <%s> is missing port %d/%d\n",
+				LILV_ERRORF("Plugin <%s> is missing port %u/%u\n",
 				            lilv_node_as_uri(plugin->plugin_uri), i, plugin->num_ports);
 				lilv_plugin_free_ports(plugin);
 				break;
