@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2015 David Robillard <http://drobilla.net>
+  Copyright 2007-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -14,11 +14,15 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#include <stdlib.h>
-#include <string.h>
-
 #include "lilv_internal.h"
 
+#include "lilv/lilv.h"
+#include "sord/sord.h"
+#include "zix/tree.h"
+
+#include <stdbool.h>
+#include <stdlib.h>
+
 LilvPluginClass*
 lilv_plugin_class_new(LilvWorld*      world,
                       const SordNode* parent_node,
@@ -48,25 +52,25 @@ lilv_plugin_class_free(LilvPluginClass* plugin_class)
 	free(plugin_class);
 }
 
-LILV_API const LilvNode*
+const LilvNode*
 lilv_plugin_class_get_parent_uri(const LilvPluginClass* plugin_class)
 {
 	return plugin_class->parent_uri ? plugin_class->parent_uri : NULL;
 }
 
-LILV_API const LilvNode*
+const LilvNode*
 lilv_plugin_class_get_uri(const LilvPluginClass* plugin_class)
 {
 	return plugin_class->uri;
 }
 
-LILV_API const LilvNode*
+const LilvNode*
 lilv_plugin_class_get_label(const LilvPluginClass* plugin_class)
 {
 	return plugin_class->label;
 }
 
-LILV_API LilvPluginClasses*
+LilvPluginClasses*
 lilv_plugin_class_get_children(const LilvPluginClass* plugin_class)
 {
 	// Returned list doesn't own categories
