@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2014 David Robillard <http://drobilla.net>
+  Copyright 2007-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -16,6 +16,10 @@
 
 #include "lilv_internal.h"
 
+#include "lilv/lilv.h"
+
+#include <stdlib.h>
+
 /** Ownership of value and label is taken */
 LilvScalePoint*
 lilv_scale_point_new(LilvNode* value, LilvNode* label)
@@ -36,13 +40,13 @@ lilv_scale_point_free(LilvScalePoint* point)
 	}
 }
 
-LILV_API const LilvNode*
+const LilvNode*
 lilv_scale_point_get_value(const LilvScalePoint* point)
 {
 	return point->value;
 }
 
-LILV_API const LilvNode*
+const LilvNode*
 lilv_scale_point_get_label(const LilvScalePoint* point)
 {
 	return point->label;
