@@ -14,6 +14,7 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
+#include "filesystem.h"
 #include "lilv_internal.h"
 
 #include "lilv/lilv.h"
@@ -223,6 +224,12 @@ retrieve_callback(LV2_State_Handle handle,
 	return NULL;
 }
 
+static bool
+path_exists(const char* path, const void* ignored)
+{
+	return lilv_path_exists(path);
+}
+
 static bool
 lilv_state_has_path(const char* path, const void* state)
 {
@@ -233,7 +240,7 @@ static char*
 make_path(LV2_State_Make_Path_Handle handle, const char* path)
 {
 	LilvState* state = (LilvState*)handle;
-	lilv_mkdir_p(state->dir);
+	lilv_create_directories(state->dir);
 
 	return lilv_path_join(state->dir, path);
 }
@@ -244,7 +251,7 @@ abstract_path(LV2_State_Map_Path_Handle handle,
 {
 	LilvState*    state     = (LilvState*)handle;
 	char*         path      = NULL;
-	char*         real_path = lilv_realpath(abs_path);
+	char*         real_path = lilv_path_canonical(abs_path);
 	const PathMap key       = { real_path, NULL };
 	ZixTreeIter*  iter      = NULL;
 
@@ -262,7 +269,7 @@ abstract_path(LV2_State_Map_Path_Handle handle,
 		// File created by plugin earlier
 		path = lilv_path_relative_to(real_path, state->scratch_dir);
 		if (state->copy_dir) {
-			int st = lilv_mkdir_p(state->copy_dir);
+			int st = lilv_create_directories(state->copy_dir);
 			if (st) {
 				LILV_ERRORF("Error creating directory %s (%s)\n",
 				            state->copy_dir, strerror(st));
@@ -273,7 +280,7 @@ abstract_path(LV2_State_Map_Path_Handle handle,
 			if (!copy || !lilv_file_equals(real_path, copy)) {
 				// No recent enough copy, make a new one
 				free(copy);
-				copy = lilv_find_free_path(cpath, lilv_path_exists, NULL);
+				copy = lilv_find_free_path(cpath, path_exists, NULL);
 				if ((st = lilv_copy_file(real_path, copy))) {
 					LILV_ERRORF("Error copying state file %s (%s)\n",
 					            copy, strerror(st));
@@ -327,23 +334,34 @@ absolute_path(LV2_State_Map_Path_Handle handle,
 	return path;
 }
 
-/** Return a new features array which is `feature` added to `features`. */
+/** Return a new features array with built-in features added to `features`. */
 static const LV2_Feature**
-add_features(const LV2_Feature *const * features,
-             const LV2_Feature* map, const LV2_Feature* make)
+add_features(const LV2_Feature* const* features,
+             const LV2_Feature*        map,
+             const LV2_Feature*        make,
+             const LV2_Feature*        free)
 {
 	size_t n_features = 0;
 	for (; features && features[n_features]; ++n_features) {}
 
 	const LV2_Feature** ret = (const LV2_Feature**)calloc(
-		n_features + 3, sizeof(LV2_Feature*));
+		n_features + 4, sizeof(LV2_Feature*));
 
 	if (features) {
 		memcpy(ret, features, n_features * sizeof(LV2_Feature*));
 	}
 
-	ret[n_features]     = map;
-	ret[n_features + 1] = make;
+	size_t i = n_features;
+	if (map) {
+		ret[i++] = map;
+	}
+	if (make) {
+		ret[i++] = make;
+	}
+	if (free) {
+		ret[i++] = free;
+	}
+
 	return ret;
 }
 
@@ -369,7 +387,13 @@ state_strerror(LV2_State_Status st)
 	}
 }
 
-LILV_API LilvState*
+static void
+lilv_free_path(LV2_State_Free_Path_Handle handle, char* path)
+{
+	lilv_free(path);
+}
+
+LilvState*
 lilv_state_new_from_instance(const LilvPlugin*          plugin,
                              LilvInstance*              instance,
                              LV2_URID_Map*              map,
@@ -398,8 +422,11 @@ lilv_state_new_from_instance(const LilvPlugin*          plugin,
 	LV2_Feature         pmap_feature  = { LV2_STATE__mapPath, &pmap };
 	LV2_State_Make_Path pmake         = { state, make_path };
 	LV2_Feature         pmake_feature = { LV2_STATE__makePath, &pmake };
+	LV2_State_Free_Path pfree         = { NULL, lilv_free_path };
+	LV2_Feature         pfree_feature = { LV2_STATE__freePath, &pfree };
 	features = sfeatures = add_features(features, &pmap_feature,
-	                                    save_dir ? &pmake_feature : NULL);
+	                                    save_dir ? &pmake_feature : NULL,
+	                                    &pfree_feature);
 
 	// Store port values
 	if (get_value) {
@@ -409,7 +436,8 @@ lilv_state_new_from_instance(const LilvPlugin*          plugin,
 			const LilvPort* const port = plugin->ports[i];
 			if (lilv_port_is_a(plugin, port, lv2_ControlPort)
 			    && lilv_port_is_a(plugin, port, lv2_InputPort)) {
-				uint32_t size, type;
+				uint32_t    size  = 0;
+				uint32_t    type  = 0;
 				const char* sym   = lilv_node_as_string(port->symbol);
 				const void* value = get_value(sym, user_data, &size, &type);
 				append_port_value(state, sym, value, size, type);
@@ -444,7 +472,7 @@ lilv_state_new_from_instance(const LilvPlugin*          plugin,
 	return state;
 }
 
-LILV_API void
+void
 lilv_state_emit_port_values(const LilvState*     state,
                             LilvSetPortValueFunc set_value,
                             void*                user_data)
@@ -456,7 +484,7 @@ lilv_state_emit_port_values(const LilvState*     state,
 	}
 }
 
-LILV_API void
+void
 lilv_state_restore(const LilvState*           state,
                    LilvInstance*              instance,
                    LilvSetPortValueFunc       set_value,
@@ -473,6 +501,9 @@ lilv_state_restore(const LilvState*           state,
 		(LilvState*)state, abstract_path, absolute_path };
 	LV2_Feature map_feature = { LV2_STATE__mapPath, &map_path };
 
+	LV2_State_Free_Path free_path    = { NULL, lilv_free_path };
+	LV2_Feature         free_feature = { LV2_STATE__freePath, &free_path };
+
 	if (instance) {
 		const LV2_Descriptor* desc = instance->lv2_descriptor;
 		if (desc->extension_data) {
@@ -481,7 +512,7 @@ lilv_state_restore(const LilvState*           state,
 
 			if (iface && iface->restore) {
 				const LV2_Feature** sfeatures = add_features(
-					features, &map_feature, NULL);
+					features, &map_feature, NULL, &free_feature);
 
 				iface->restore(instance->lv2_handle, retrieve_callback,
 				               (LV2_State_Handle)state, flags, sfeatures);
@@ -501,8 +532,11 @@ static void
 set_state_dir_from_model(LilvState* state, const SordNode* graph)
 {
 	if (!state->dir && graph) {
-		const char* uri = (const char*)sord_node_get_string(graph);
-		state->dir = lilv_file_uri_parse(uri, NULL);
+		const char* uri  = (const char*)sord_node_get_string(graph);
+		char*       path = lilv_file_uri_parse(uri, NULL);
+
+		state->dir = lilv_path_join(path, NULL);
+		free(path);
 	}
 	assert(!state->dir || lilv_path_is_absolute(state->dir));
 }
@@ -521,7 +555,7 @@ new_state_from_model(LilvWorld*       world,
 
 	// Allocate state
 	LilvState* const state = (LilvState*)calloc(1, sizeof(LilvState));
-	state->dir       = lilv_strdup(dir);
+	state->dir       = lilv_path_join(dir, NULL);
 	state->atom_Path = map->map(map->handle, LV2_ATOM__Path);
 	state->uri       = lilv_node_new_from_node(world, node);
 
@@ -648,7 +682,7 @@ new_state_from_model(LilvWorld*       world,
 	return state;
 }
 
-LILV_API LilvState*
+LilvState*
 lilv_state_new_from_world(LilvWorld*      world,
                           LV2_URID_Map*   map,
                           const LilvNode* node)
@@ -662,7 +696,7 @@ lilv_state_new_from_world(LilvWorld*      world,
 	return new_state_from_model(world, map, world->model, node->node, NULL);
 }
 
-LILV_API LilvState*
+LilvState*
 lilv_state_new_from_file(LilvWorld*      world,
                          LV2_URID_Map*   map,
                          const LilvNode* subject,
@@ -687,12 +721,14 @@ lilv_state_new_from_file(LilvWorld*      world,
 		? subject->node
 		: sord_node_from_serd_node(world->world, env, &node, NULL, NULL);
 
-	char* dirname   = lilv_dirname(path);
-	char* real_path = lilv_realpath(dirname);
-	LilvState* state = new_state_from_model(
-		world, map, model, subject_node, real_path);
-	free(dirname);
+	char*      dirname   = lilv_path_parent(path);
+	char*      real_path = lilv_path_canonical(dirname);
+	char*      dir_path  = lilv_path_join(real_path, NULL);
+	LilvState* state =
+		new_state_from_model(world, map, model, subject_node, dir_path);
+	free(dir_path);
 	free(real_path);
+	free(dirname);
 
 	serd_node_free(&node);
 	free(abs_path);
@@ -715,7 +751,7 @@ set_prefixes(SerdEnv* env)
 	SET_PSET(env, USTR("xsd"),   USTR(LILV_NS_XSD));
 }
 
-LILV_API LilvState*
+LilvState*
 lilv_state_new_from_string(LilvWorld*    world,
                            LV2_URID_Map* map,
                            const char*   str)
@@ -821,6 +857,30 @@ remove_manifest_entry(SordWorld* world, SordModel* model, const char* subject)
 	sord_node_free(world, s);
 }
 
+static int
+write_manifest(LilvWorld*      world,
+               SerdEnv*        env,
+               SordModel*      model,
+               const SerdNode* file_uri)
+{
+	char* const path = (char*)serd_file_uri_parse(file_uri->buf, NULL);
+	FILE* const wfd  = fopen(path, "w");
+	if (!wfd) {
+		LILV_ERRORF("Failed to open %s for writing (%s)\n",
+		            path, strerror(errno));
+
+		serd_free(path);
+		return 1;
+	}
+
+	SerdWriter* writer = ttl_file_writer(wfd, file_uri, &env);
+	sord_write(model, writer, NULL);
+	serd_writer_free(writer);
+	fclose(wfd);
+	serd_free(path);
+	return 0;
+}
+
 static int
 add_state_to_manifest(LilvWorld*      lworld,
                       const LilvNode* plugin_uri,
@@ -838,7 +898,7 @@ add_state_to_manifest(LilvWorld*      lworld,
 	if (rfd) {
 		// Read manifest into model
 		SerdReader* reader = sord_new_reader(model, env, SERD_TURTLE, NULL);
-		lilv_flock(rfd, true);
+		lilv_flock(rfd, true, true);
 		serd_reader_read_file_handle(reader, rfd, manifest.buf);
 		serd_reader_free(reader);
 	}
@@ -880,16 +940,7 @@ add_state_to_manifest(LilvWorld*      lworld,
 	                                   USTR(lilv_node_as_string(plugin_uri))));
 
 	// Write manifest model to file
-	FILE* wfd = fopen(manifest_path, "w");
-	if (wfd) {
-		SerdWriter* writer = ttl_file_writer(wfd, &manifest, &env);
-		sord_write(model, writer, NULL);
-		serd_writer_free(writer);
-		fclose(wfd);
-	} else {
-		LILV_ERRORF("Failed to open %s for writing (%s)\n",
-		            manifest_path, strerror(errno));
-	}
+	write_manifest(lworld, env, model, &manifest);
 
 	sord_free(model);
 	serd_node_free(&file);
@@ -897,7 +948,7 @@ add_state_to_manifest(LilvWorld*      lworld,
 	serd_env_free(env);
 
 	if (rfd) {
-		lilv_flock(rfd, false);
+		lilv_flock(rfd, false, true);
 		fclose(rfd);
 	}
 
@@ -908,10 +959,10 @@ static bool
 link_exists(const char* path, const void* data)
 {
 	const char* target = (const char*)data;
-	if (!lilv_path_exists(path, NULL)) {
+	if (!lilv_path_exists(path)) {
 		return false;
 	}
-	char* real_path = lilv_realpath(path);
+	char* real_path = lilv_path_canonical(path);
 	bool  matches   = !strcmp(real_path, target);
 	free(real_path);
 	return !matches;
@@ -920,7 +971,19 @@ link_exists(const char* path, const void* data)
 static int
 maybe_symlink(const char* oldpath, const char* newpath)
 {
-  return link_exists(newpath, oldpath) ? 0 : lilv_symlink(oldpath, newpath);
+	if (link_exists(newpath, oldpath)) {
+		return 0;
+	}
+
+	const int st = lilv_symlink(oldpath, newpath);
+	if (st) {
+		LILV_ERRORF("Failed to link %s => %s (%s)\n",
+		            newpath,
+		            oldpath,
+		            strerror(errno));
+	}
+
+	return st;
 }
 
 static void
@@ -1077,8 +1140,13 @@ lilv_state_make_links(const LilvState* state, const char* dir)
 			} else {
 				// Make a link in the link directory to external file
 				char* lpath = lilv_find_free_path(pat, link_exists, pm->abs);
-				if (!lilv_path_exists(lpath, NULL)) {
-					lilv_symlink(pm->abs, lpath);
+				if (!lilv_path_exists(lpath)) {
+					if (lilv_symlink(pm->abs, lpath)) {
+						LILV_ERRORF("Failed to link %s => %s (%s)\n",
+						            pm->abs,
+						            lpath,
+						            strerror(errno));
+					}
 				}
 
 				// Make a link in the save directory to the external link
@@ -1093,7 +1161,7 @@ lilv_state_make_links(const LilvState* state, const char* dir)
 	}
 }
 
-LILV_API int
+int
 lilv_state_save(LilvWorld*       world,
                 LV2_URID_Map*    map,
                 LV2_URID_Unmap*  unmap,
@@ -1102,7 +1170,7 @@ lilv_state_save(LilvWorld*       world,
                 const char*      dir,
                 const char*      filename)
 {
-	if (!filename || !dir || lilv_mkdir_p(dir)) {
+	if (!filename || !dir || lilv_create_directories(dir)) {
 		return 1;
 	}
 
@@ -1148,7 +1216,7 @@ lilv_state_save(LilvWorld*       world,
 	return ret;
 }
 
-LILV_API char*
+char*
 lilv_state_to_string(LilvWorld*       world,
                      LV2_URID_Map*    map,
                      LV2_URID_Unmap*  unmap,
@@ -1177,18 +1245,20 @@ lilv_state_to_string(LilvWorld*       world,
 }
 
 static void
-try_unlink(const char* path)
+try_unlink(const char* state_dir, const char* path)
 {
-	if (unlink(path)) {
-		LILV_ERRORF("Failed to remove %s (%s)\n", path, strerror(errno));
+	if (!strncmp(state_dir, path, strlen(state_dir))) {
+		if (lilv_path_exists(path) && lilv_remove(path)) {
+			LILV_ERRORF("Failed to remove %s (%s)\n", path, strerror(errno));
+		}
 	}
 }
 
-LILV_API int
+int
 lilv_state_delete(LilvWorld*       world,
                   const LilvState* state)
 {
-	if (!state->dir || !state->uri) {
+	if (!state->dir) {
 		LILV_ERROR("Attempt to delete unsaved state\n");
 		return -1;
 	}
@@ -1196,9 +1266,10 @@ lilv_state_delete(LilvWorld*       world,
 	LilvNode*  bundle        = lilv_new_file_uri(world, NULL, state->dir);
 	LilvNode*  manifest      = lilv_world_get_manifest_uri(world, bundle);
 	char*      manifest_path = lilv_node_get_path(manifest, NULL);
+	const bool has_manifest  = lilv_path_exists(manifest_path);
 	SordModel* model         = sord_new(world->world, SORD_SPO, false);
 
-	{
+	if (has_manifest) {
 		// Read manifest into temporary local model
 		SerdEnv*    env = serd_env_new(sord_node_to_serd_node(manifest->node));
 		SerdReader* ttl = sord_new_reader(model, env, SERD_TURTLE, NULL);
@@ -1207,46 +1278,65 @@ lilv_state_delete(LilvWorld*       world,
 		serd_env_free(env);
 	}
 
-	SordNode* file = sord_get(
-		model, state->uri->node, world->uris.rdfs_seeAlso, NULL, NULL);
-	if (file) {
-		// Remove state file
-		char* path =
-			(char*)serd_file_uri_parse(sord_node_get_string(file), NULL);
-		try_unlink(path);
-		serd_free(path);
-	}
+	if (state->uri) {
+		SordNode* file = sord_get(
+			model, state->uri->node, world->uris.rdfs_seeAlso, NULL, NULL);
+		if (file) {
+			// Remove state file
+			const uint8_t* uri  = sord_node_get_string(file);
+			char*          path = (char*)serd_file_uri_parse(uri, NULL);
+			try_unlink(state->dir, path);
+			serd_free(path);
+		}
 
-	// Remove any existing manifest entries for this state
-	remove_manifest_entry(
-		world->world, model, lilv_node_as_string(state->uri));
-	remove_manifest_entry(
-		world->world, world->model, lilv_node_as_string(state->uri));
+		// Remove any existing manifest entries for this state
+		const char* state_uri_str = lilv_node_as_string(state->uri);
+		remove_manifest_entry(world->world, model, state_uri_str);
+		remove_manifest_entry(world->world, world->model, state_uri_str);
+	}
 
 	// Drop bundle from model
 	lilv_world_unload_bundle(world, bundle);
 
 	if (sord_num_quads(model) == 0) {
 		// Manifest is empty, attempt to remove bundle entirely
-		try_unlink(manifest_path);
-
-		// Remove all known files from state bundle
-		for (ZixTreeIter* i = zix_tree_begin(state->abs2rel);
-		     i != zix_tree_end(state->abs2rel);
-		     i = zix_tree_iter_next(i)) {
-			const PathMap* pm   = (const PathMap*)zix_tree_get(i);
-			char*          path = lilv_path_join(state->dir, pm->rel);
-			try_unlink(path);
-			free(path);
+		if (has_manifest) {
+			try_unlink(state->dir, manifest_path);
 		}
 
-		if (rmdir(state->dir)) {
+		// Remove all known files from state bundle
+		if (state->abs2rel) {
+			// State created from instance, get paths from map
+			for (ZixTreeIter* i = zix_tree_begin(state->abs2rel);
+			     i != zix_tree_end(state->abs2rel);
+			     i = zix_tree_iter_next(i)) {
+				const PathMap* pm   = (const PathMap*)zix_tree_get(i);
+				char*          path = lilv_path_join(state->dir, pm->rel);
+				try_unlink(state->dir, path);
+				free(path);
+			}
+		} else {
+			// State loaded from model, get paths from loaded properties
+			for (uint32_t i = 0; i < state->props.n; ++i) {
+				const Property* const p = &state->props.props[i];
+				if (p->type == state->atom_Path) {
+					try_unlink(state->dir, (const char*)p->value);
+				}
+			}
+		}
+
+		if (lilv_remove(state->dir)) {
 			LILV_ERRORF("Failed to remove directory %s (%s)\n",
 			            state->dir, strerror(errno));
 		}
 	} else {
-		// Still something in the manifest, reload bundle
+		// Still something in the manifest, update and reload bundle
+		const SerdNode* manifest_node = sord_node_to_serd_node(manifest->node);
+		SerdEnv*        env           = serd_env_new(manifest_node);
+
+		write_manifest(world, env, model, manifest_node);
 		lilv_world_load_bundle(world, bundle);
+		serd_env_free(env);
 	}
 
 	sord_free(model);
@@ -1270,7 +1360,7 @@ free_property_array(LilvState* state, PropertyArray* array)
 	free(array->props);
 }
 
-LILV_API void
+void
 lilv_state_free(LilvState* state)
 {
 	if (state) {
@@ -1294,7 +1384,7 @@ lilv_state_free(LilvState* state)
 	}
 }
 
-LILV_API bool
+bool
 lilv_state_equals(const LilvState* a, const LilvState* b)
 {
 	if (!lilv_node_equals(a->plugin_uri, b->plugin_uri)
@@ -1338,31 +1428,31 @@ lilv_state_equals(const LilvState* a, const LilvState* b)
 	return true;
 }
 
-LILV_API unsigned
+unsigned
 lilv_state_get_num_properties(const LilvState* state)
 {
 	return state->props.n;
 }
 
-LILV_API const LilvNode*
+const LilvNode*
 lilv_state_get_plugin_uri(const LilvState* state)
 {
 	return state->plugin_uri;
 }
 
-LILV_API const LilvNode*
+const LilvNode*
 lilv_state_get_uri(const LilvState* state)
 {
 	return state->uri;
 }
 
-LILV_API const char*
+const char*
 lilv_state_get_label(const LilvState* state)
 {
 	return state->label;
 }
 
-LILV_API void
+void
 lilv_state_set_label(LilvState* state, const char* label)
 {
 	const size_t len = strlen(label);
@@ -1370,7 +1460,7 @@ lilv_state_set_label(LilvState* state, const char* label)
 	memcpy(state->label, label, len + 1);
 }
 
-LILV_API int
+int
 lilv_state_set_metadata(LilvState*  state,
                         uint32_t    key,
                         const void* value,
