@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2016 David Robillard <http://drobilla.net>
+  Copyright 2007-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -14,22 +14,38 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
+#include "filesystem.h"
+#include "lilv_config.h"
+#include "lilv_internal.h"
+
+#include "lilv/lilv.h"
+#include "serd/serd.h"
+#include "sord/sord.h"
+#include "zix/common.h"
+#include "zix/tree.h"
+
+#include "lv2/core/lv2.h"
+#include "lv2/presets/presets.h"
+
+#ifdef LILV_DYN_MANIFEST
+#    include "lv2/dynmanifest/dynmanifest.h"
+#    include <dlfcn.h>
+#endif
+
 #include <assert.h>
-#include <errno.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "lv2/lv2plug.in/ns/ext/presets/presets.h"
-
-#include "lilv_internal.h"
-
 static int
 lilv_world_drop_graph(LilvWorld* world, const SordNode* graph);
 
-LILV_API LilvWorld*
+LilvWorld*
 lilv_world_new(void)
 {
-	LilvWorld* world = (LilvWorld*)malloc(sizeof(LilvWorld));
+	LilvWorld* world = (LilvWorld*)calloc(1, sizeof(LilvWorld));
 
 	world->world = sord_world_new();
 	if (!world->world) {
@@ -110,7 +126,7 @@ fail:
 	return NULL;
 }
 
-LILV_API void
+void
 lilv_world_free(LilvWorld* world)
 {
 	if (!world) {
@@ -163,10 +179,11 @@ lilv_world_free(LilvWorld* world)
 	sord_world_free(world->world);
 	world->world = NULL;
 
+	free(world->opt.lv2_path);
 	free(world);
 }
 
-LILV_API void
+void
 lilv_world_set_option(LilvWorld*      world,
                       const char*     uri,
                       const LilvNode* value)
@@ -181,11 +198,16 @@ lilv_world_set_option(LilvWorld*      world,
 			world->opt.filter_language = lilv_node_as_bool(value);
 			return;
 		}
+	} else if (!strcmp(uri, LILV_OPTION_LV2_PATH)) {
+		if (lilv_node_is_string(value)) {
+			world->opt.lv2_path = lilv_strdup(lilv_node_as_string(value));
+			return;
+		}
 	}
 	LILV_WARNF("Unrecognized or invalid option `%s'\n", uri);
 }
 
-LILV_API LilvNodes*
+LilvNodes*
 lilv_world_find_nodes(LilvWorld*      world,
                       const LilvNode* subject,
                       const LilvNode* predicate,
@@ -213,12 +235,32 @@ lilv_world_find_nodes(LilvWorld*      world,
 	                                      object ? object->node : NULL);
 }
 
-LILV_API LilvNode*
+LilvNode*
 lilv_world_get(LilvWorld*      world,
                const LilvNode* subject,
                const LilvNode* predicate,
                const LilvNode* object)
 {
+	if (!object) {
+		// TODO: Improve performance (see lilv_plugin_get_one)
+		SordIter* stream = sord_search(world->model,
+		                               subject   ? subject->node : NULL,
+		                               predicate ? predicate->node : NULL,
+		                               NULL,
+		                               NULL);
+
+		LilvNodes* nodes =
+		    lilv_nodes_from_stream_objects(world, stream, SORD_OBJECT);
+
+		if (nodes) {
+			LilvNode* value = lilv_node_duplicate(lilv_nodes_get_first(nodes));
+			lilv_nodes_free(nodes);
+			return value;
+		}
+
+		return NULL;
+	}
+
 	SordNode* snode = sord_get(world->model,
 	                           subject   ? subject->node   : NULL,
 	                           predicate ? predicate->node : NULL,
@@ -247,7 +289,7 @@ lilv_world_ask_internal(LilvWorld*      world,
 	return sord_ask(world->model, subject, predicate, object, NULL);
 }
 
-LILV_API bool
+bool
 lilv_world_ask(LilvWorld*      world,
                const LilvNode* subject,
                const LilvNode* predicate,
@@ -304,7 +346,7 @@ const uint8_t*
 lilv_world_blank_node_prefix(LilvWorld* world)
 {
 	static char str[32];
-	snprintf(str, sizeof(str), "%d", world->n_read_files++);
+	snprintf(str, sizeof(str), "%u", world->n_read_files++);
 	return (const uint8_t*)str;
 }
 
@@ -556,7 +598,7 @@ lilv_world_load_dyn_manifest(LilvWorld*      world,
 			continue;
 		}
 
-		LilvDynManifest* desc = malloc(sizeof(LilvDynManifest));
+		LilvDynManifest* desc = (LilvDynManifest*)malloc(sizeof(LilvDynManifest));
 		desc->bundle = lilv_node_new_from_node(world, bundle_node);
 		desc->lib    = lib;
 		desc->handle = handle;
@@ -598,7 +640,7 @@ lilv_world_load_dyn_manifest(LilvWorld*      world,
 			lilv_world_add_plugin(world, plug, manifest, desc, bundle_node);
 		}
 		if (desc->refs == 0) {
-			free(desc);
+			lilv_dynmanifest_free(desc);
 		}
 		sord_iter_free(p);
 		sord_free(plugins);
@@ -609,6 +651,23 @@ lilv_world_load_dyn_manifest(LilvWorld*      world,
 #endif  // LILV_DYN_MANIFEST
 }
 
+#ifdef LILV_DYN_MANIFEST
+void
+lilv_dynmanifest_free(LilvDynManifest* dynmanifest)
+{
+	typedef int (*CloseFunc)(LV2_Dyn_Manifest_Handle);
+	CloseFunc close_func = (CloseFunc)lilv_dlfunc(dynmanifest->lib,
+	                                              "lv2_dyn_manifest_close");
+	if (close_func) {
+		close_func(dynmanifest->handle);
+	}
+
+	dlclose(dynmanifest->lib);
+	lilv_node_free(dynmanifest->bundle);
+	free(dynmanifest);
+}
+#endif  // LILV_DYN_MANIFEST
+
 LilvNode*
 lilv_world_get_manifest_uri(LilvWorld* world, const LilvNode* bundle_uri)
 {
@@ -679,7 +738,7 @@ get_version(LilvWorld* world, SordModel* model, const LilvNode* subject)
 	return version;
 }
 
-LILV_API void
+void
 lilv_world_load_bundle(LilvWorld* world, const LilvNode* bundle_uri)
 {
 	if (!lilv_node_is_uri(bundle_uri)) {
@@ -834,7 +893,7 @@ lilv_world_drop_graph(LilvWorld* world, const SordNode* graph)
 static int
 lilv_world_unload_file(LilvWorld* world, const LilvNode* file)
 {
-	ZixTreeIter* iter;
+	ZixTreeIter* iter = NULL;
 	if (!zix_tree_find((ZixTree*)world->loaded_files, file, &iter)) {
 		zix_tree_remove((ZixTree*)world->loaded_files, iter);
 		return 0;
@@ -842,7 +901,7 @@ lilv_world_unload_file(LilvWorld* world, const LilvNode* file)
 	return 1;
 }
 
-LILV_API int
+int
 lilv_world_unload_bundle(LilvWorld* world, const LilvNode* bundle_uri)
 {
 	if (!bundle_uri) {
@@ -897,13 +956,9 @@ static void
 load_dir_entry(const char* dir, const char* name, void* data)
 {
 	LilvWorld* world = (LilvWorld*)data;
-	if (!strcmp(name, ".") || !strcmp(name, "..")) {
-		return;
-	}
-
-	char*     path = lilv_strjoin(dir, "/", name, "/", NULL);
-	SerdNode  suri = serd_node_new_file_uri((const uint8_t*)path, 0, 0, true);
-	LilvNode* node = lilv_new_uri(world, (const char*)suri.buf);
+	char*      path  = lilv_strjoin(dir, "/", name, "/", NULL);
+	SerdNode   suri  = serd_node_new_file_uri((const uint8_t*)path, 0, 0, true);
+	LilvNode*  node  = lilv_new_uri(world, (const char*)suri.buf);
 
 	lilv_world_load_bundle(world, node);
 	lilv_node_free(node);
@@ -1013,10 +1068,13 @@ lilv_world_load_plugin_classes(LilvWorld* world)
 	sord_iter_free(classes);
 }
 
-LILV_API void
+void
 lilv_world_load_all(LilvWorld* world)
 {
-	const char* lv2_path = getenv("LV2_PATH");
+	const char* lv2_path = world->opt.lv2_path;
+	if (!lv2_path) {
+		lv2_path = getenv("LV2_PATH");
+	}
 	if (!lv2_path) {
 		lv2_path = LILV_DEFAULT_LV2_PATH;
 	}
@@ -1047,12 +1105,12 @@ lilv_world_load_all(LilvWorld* world)
 SerdStatus
 lilv_world_load_file(LilvWorld* world, SerdReader* reader, const LilvNode* uri)
 {
-	ZixTreeIter* iter;
+	ZixTreeIter* iter = NULL;
 	if (!zix_tree_find((ZixTree*)world->loaded_files, uri, &iter)) {
 		return SERD_FAILURE;  // File has already been loaded
 	}
 
-	size_t               uri_len;
+	size_t               uri_len = 0;
 	const uint8_t* const uri_str = sord_node_get_string_counted(
 		uri->node, &uri_len);
 	if (strncmp((const char*)uri_str, "file:", 5)) {
@@ -1074,7 +1132,7 @@ lilv_world_load_file(LilvWorld* world, SerdReader* reader, const LilvNode* uri)
 	return SERD_SUCCESS;
 }
 
-LILV_API int
+int
 lilv_world_load_resource(LilvWorld*      world,
                          const LilvNode* resource)
 {
@@ -1109,7 +1167,7 @@ lilv_world_load_resource(LilvWorld*      world,
 	return n_read;
 }
 
-LILV_API int
+int
 lilv_world_unload_resource(LilvWorld*      world,
                            const LilvNode* resource)
 {
@@ -1145,25 +1203,25 @@ lilv_world_unload_resource(LilvWorld*      world,
 	return n_dropped;
 }
 
-LILV_API const LilvPluginClass*
+const LilvPluginClass*
 lilv_world_get_plugin_class(const LilvWorld* world)
 {
 	return world->lv2_plugin_class;
 }
 
-LILV_API const LilvPluginClasses*
+const LilvPluginClasses*
 lilv_world_get_plugin_classes(const LilvWorld* world)
 {
 	return world->plugin_classes;
 }
 
-LILV_API const LilvPlugins*
+const LilvPlugins*
 lilv_world_get_all_plugins(const LilvWorld* world)
 {
 	return world->plugins;
 }
 
-LILV_API LilvNode*
+LilvNode*
 lilv_world_get_symbol(LilvWorld* world, const LilvNode* subject)
 {
 	// Check for explicitly given symbol
