@@ -1,6 +1,6 @@
 /*
   Lilv Test Plugin - Failed lib descriptor
-  Copyright 2011-2015 David Robillard <d@drobilla.net>
+  Copyright 2011-2019 David Robillard <d@drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -15,12 +15,10 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
+#include "lv2/core/lv2.h"
+
 #include <stdlib.h>
 
-#include "lv2/lv2plug.in/ns/lv2core/lv2.h"
-
-#define PLUGIN_URI "http://example.org/failed-lib-descriptor"
-
 LV2_SYMBOL_EXPORT
 const LV2_Lib_Descriptor*
 lv2_lib_descriptor(const char*              bundle_path,
