@@ -1,15 +1,18 @@
+#undef NDEBUG
+
+#include "../src/filesystem.h"
+
+#include "serd/serd.h"
 #include "lilv/lilv.h"
-#include "../src/lilv_internal.h"
+
+#include <assert.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
 
 #define PLUGIN_URI "http://example.org/failed-lib-descriptor"
 
-#define TEST_ASSERT(check) do {\
-	if (!(check)) {\
-		fprintf(stderr, "%s:%d: failed test: %s\n", __FILE__, __LINE__, #check);\
-		return 1;\
-	}\
-} while (0)
-
 int
 main(int argc, char** argv)
 {
@@ -28,15 +31,17 @@ main(int argc, char** argv)
 	lilv_world_load_bundle(world, bundle_uri);
 	free(abs_bundle);
 	serd_node_free(&bundle);
+	lilv_node_free(bundle_uri);
 
 	LilvNode*          plugin_uri = lilv_new_uri(world, PLUGIN_URI);
 	const LilvPlugins* plugins    = lilv_world_get_all_plugins(world);
 	const LilvPlugin*  plugin     = lilv_plugins_get_by_uri(plugins, plugin_uri);
-	TEST_ASSERT(plugin);
+	assert(plugin);
 
 	LilvInstance* instance = lilv_plugin_instantiate(plugin, 48000.0, NULL);
-	TEST_ASSERT(!instance);
+	assert(!instance);
 
+	lilv_node_free(plugin_uri);
 	lilv_world_free(world);
 
 	return 0;
