@@ -1,6 +1,6 @@
 /*
   Lilv Test Plugin - Missing plugin
-  Copyright 2011-2015 David Robillard <d@drobilla.net>
+  Copyright 2011-2019 David Robillard <d@drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -15,14 +15,28 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
+#include "lv2/core/lv2.h"
+
+#include <stdint.h>
 #include <stdlib.h>
 
-#include "lv2/lv2plug.in/ns/lv2core/lv2.h"
-
-#define PLUGIN_URI "http://example.org/missing-plugin"
+static const LV2_Descriptor descriptor = {
+	"http://example.org/not-the-plugin-you-are-looking-for",
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL,
+	NULL
+};
 
 LV2_SYMBOL_EXPORT
 const LV2_Descriptor* lv2_descriptor(uint32_t index)
 {
+	if (index == 0) {
+		return &descriptor;
+	}
+
 	return NULL;
 }
