@@ -1,5 +1,5 @@
 /*
-  Copyright 2012-2017 David Robillard <http://drobilla.net>
+  Copyright 2012-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -16,17 +16,22 @@
 
 #define _POSIX_C_SOURCE 200809L
 
+#include "lilv/lilv.h"
+#include "lv2/atom/atom.h"
+#include "lv2/core/lv2.h"
+#include "lv2/urid/urid.h"
+
+#include "bench.h"
+#include "lilv_config.h"
+#include "uri_table.h"
+
 #include <math.h>
+#include <stdbool.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-
-#include "lilv/lilv.h"
-#include "lv2/lv2plug.in/ns/ext/atom/atom.h"
-
-#include "lilv_config.h"
-#include "bench.h"
-#include "uri_table.h"
+#include <time.h>
 
 static LilvNode* atom_AtomPort   = NULL;
 static LilvNode* atom_Sequence   = NULL;
@@ -44,7 +49,7 @@ print_version(void)
 {
 	printf(
 		"lv2bench (lilv) " LILV_VERSION "\n"
-		"Copyright 2012-2017 David Robillard <http://drobilla.net>\n"
+		"Copyright 2012-2019 David Robillard <http://drobilla.net>\n"
 		"License: <http://www.opensource.org/licenses/isc-license>\n"
 		"This is free software: you are free to change and redistribute it.\n"
 		"There is NO WARRANTY, to the extent permitted by law.\n");
@@ -141,7 +146,7 @@ bench(const LilvPlugin* p, uint32_t sample_count, uint32_t block_size)
 			} else if (lilv_port_is_a(p, port, lv2_OutputPort)) {
 				lilv_instance_connect_port(instance, index, out);
 			} else {
-				fprintf(stderr, "<%s> port %d neither input nor output, skipping\n",
+				fprintf(stderr, "<%s> port %u neither input nor output, skipping\n",
 				        uri, index);
 				lilv_instance_free(instance);
 				free(seq_out);
@@ -157,7 +162,7 @@ bench(const LilvPlugin* p, uint32_t sample_count, uint32_t block_size)
 				lilv_instance_connect_port(instance, index, seq_out);
 			}
 		} else {
-			fprintf(stderr, "<%s> port %d has unknown type, skipping\n",
+			fprintf(stderr, "<%s> port %u has unknown type, skipping\n",
 			        uri, index);
 			lilv_instance_free(instance);
 			free(seq_out);
@@ -183,17 +188,19 @@ bench(const LilvPlugin* p, uint32_t sample_count, uint32_t block_size)
 
 	lilv_instance_deactivate(instance);
 	lilv_instance_free(instance);
+	free(controls);
+	free(maxes);
+	free(mins);
 	free(seq_out);
 
 	uri_table_destroy(&uri_table);
 
 	if (full_output) {
-		printf("%d %d ", block_size, sample_count);
+		printf("%u %u ", block_size, sample_count);
 	}
 	printf("%lf %s\n", elapsed, uri);
 
 	free(buf);
-	free(controls);
 	return elapsed;
 }
 
@@ -247,6 +254,7 @@ main(int argc, char** argv)
 	if (plugin_uri_str) {
 		LilvNode* uri = lilv_new_uri(world, plugin_uri_str);
 		bench(lilv_plugins_get_by_uri(plugins, uri), sample_count, block_size);
+		lilv_node_free(uri);
 	} else {
 		LILV_FOREACH(plugins, i, plugins) {
 			bench(lilv_plugins_get(plugins, i), sample_count, block_size);
