@@ -1,5 +1,5 @@
 /*
-  Copyright 2007-2014 David Robillard <http://drobilla.net>
+  Copyright 2007-2019 David Robillard <http://drobilla.net>
 
   Permission to use, copy, modify, and/or distribute this software for any
   purpose with or without fee is hereby granted, provided that the above
@@ -14,32 +14,29 @@
   OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */
 
-#include <float.h>
+#include "lilv_config.h"
+
+#include "lilv/lilv.h"
+#include "lv2/core/lv2.h"
+#include "lv2/event/event.h"
+#include "lv2/port-groups/port-groups.h"
+#include "lv2/presets/presets.h"
+
 #include <math.h>
+#include <stdbool.h>
+#include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "lv2/lv2plug.in/ns/ext/port-groups/port-groups.h"
-#include "lv2/lv2plug.in/ns/ext/presets/presets.h"
-#include "lv2/lv2plug.in/ns/ext/event/event.h"
-
-#include "lilv/lilv.h"
-
-#include "lilv_config.h"
-
-#ifdef _MSC_VER
-#    define isnan _isnan
-#endif
-
-LilvNode* applies_to_pred     = NULL;
-LilvNode* control_class       = NULL;
-LilvNode* event_class         = NULL;
-LilvNode* group_pred          = NULL;
-LilvNode* label_pred          = NULL;
-LilvNode* preset_class        = NULL;
-LilvNode* designation_pred    = NULL;
-LilvNode* supports_event_pred = NULL;
+static LilvNode* applies_to_pred     = NULL;
+static LilvNode* control_class       = NULL;
+static LilvNode* event_class         = NULL;
+static LilvNode* group_pred          = NULL;
+static LilvNode* label_pred          = NULL;
+static LilvNode* preset_class        = NULL;
+static LilvNode* designation_pred    = NULL;
+static LilvNode* supports_event_pred = NULL;
 
 static void
 print_port(const LilvPlugin* p,
@@ -50,7 +47,7 @@ print_port(const LilvPlugin* p,
 {
 	const LilvPort* port = lilv_plugin_get_port_by_index(p, index);
 
-	printf("\n\tPort %d:\n", index);
+	printf("\n\tPort %u:\n", index);
 
 	if (!port) {
 		printf("\t\tERROR: Illegal/nonexistent port\n");
@@ -186,7 +183,7 @@ print_plugin(LilvWorld*        world,
 
 	if (lilv_plugin_has_latency(p)) {
 		uint32_t latency_port = lilv_plugin_get_latency_port_index(p);
-		printf("\tHas latency:       yes, reported by port %d\n", latency_port);
+		printf("\tHas latency:       yes, reported by port %u\n", latency_port);
 	} else {
 		printf("\tHas latency:       no\n");
 	}
@@ -338,7 +335,7 @@ print_version(void)
 {
 	printf(
 		"lv2info (lilv) " LILV_VERSION "\n"
-		"Copyright 2007-2014 David Robillard <http://drobilla.net>\n"
+		"Copyright 2007-2019 David Robillard <http://drobilla.net>\n"
 		"License: <http://www.opensource.org/licenses/isc-license>\n"
 		"This is free software: you are free to change and redistribute it.\n"
 		"There is NO WARRANTY, to the extent permitted by law.\n");
